searchNodes=[{"doc":"Isolates groups of process subtrees together for configuration and name registration purposes.Do you struggle with Elixir's global namespace for process names?Do all of your tests run synchronously because of this collisions?Do you mutate your application environment during tests?If the above problems sounds familiar, Grouper might be for you.UsageSimply start your GenServers like this:GenServer.start_link(mod, arg, name: {:via, Grouper.Registry})And access configuration like this:Grouper.Config.get(:key)In tests, scripts and IEX; initialize a group with:{:ok, _} = Grouper.start_link()Or run a single function in its own group like this:Grouper.exec!(&amp;MyApp.my_task/0)During normal application runtime, each application gets its own namespace for processes and has isolated config.During tests, however, each test can get its own group with isolated naming and configuration. This makes it trivial to run all of your tests asynchronously, eliminates the need to use global config for things like mocking, and prevents config mutations in different tests from interfering with each other.Scripts and IEX can similarly benefit from group isolation with a single command, thereby rounding out behavior to be identical in all common execution environments.MigrationFor convenience, the OTP config environment is loaded into the config for you, simplifying migration from older, global configuration. This can be suppressed if desired (see Grouper.Config.suppress_otp_env/1 for details).","ref":"Grouper.html","title":"Grouper","type":"module"},{"doc":"execute a function in its own group, returning an :ok or :error resultOptions are passed through to Grouper.start_link/1 and Grouper.stop/1.","ref":"Grouper.html#exec/2","title":"Grouper.exec/2","type":"function"},{"doc":"execute a function in its own group, returning an :ok or :error resultOptions are passed through to Grouper.start_link/1 and Grouper.stop/1.","ref":"Grouper.html#exec/4","title":"Grouper.exec/4","type":"function"},{"doc":"execute a function in its own group, raising an exception on errorOptions are passed through to Grouper.start_link/1 and Grouper.stop/1.","ref":"Grouper.html#exec!/2","title":"Grouper.exec!/2","type":"function"},{"doc":"execute a function in its own group, raising an exception on errorOptions are passed through to Grouper.start_link/1 and Grouper.stop/1.","ref":"Grouper.html#exec!/4","title":"Grouper.exec!/4","type":"function"},{"doc":"See Grouper.Group.start_link/1.","ref":"Grouper.html#start_link/1","title":"Grouper.start_link/1","type":"function"},{"doc":"See Grouper.Group.stop/1.","ref":"Grouper.html#stop/1","title":"Grouper.stop/1","type":"function"},{"doc":"Application for GrouperThis starts up an application supervisor that manages global tables and metadata used to drive groups.","ref":"Grouper.Application.html","title":"Grouper.Application","type":"module"},{"doc":"configuration functionality for groups","ref":"Grouper.Config.html","title":"Grouper.Config","type":"module"},{"doc":"delete configuration for a given key","ref":"Grouper.Config.html#del/2","title":"Grouper.Config.del/2","type":"function"},{"doc":"get configuration for a given key","ref":"Grouper.Config.html#get/2","title":"Grouper.Config.get/2","type":"function"},{"doc":"get all configuration as a keyword list","ref":"Grouper.Config.html#get_all/1","title":"Grouper.Config.get_all/1","type":"function"},{"doc":"Used to load a specific application's environment. Not needed when running under an application, but useful for scripts and tests.NOTE: Will not load the environment if it's already been loaded or suppressed.","ref":"Grouper.Config.html#load_otp_env/2","title":"Grouper.Config.load_otp_env/2","type":"function"},{"doc":"set configuration for a given key to a value","ref":"Grouper.Config.html#put/3","title":"Grouper.Config.put/3","type":"function"},{"doc":"Used when running under an application to suppress loading the application config.NOTE: Will not suppress loading the environment if it's already been loaded. Must be called before anything attempts to read config to be effective.","ref":"Grouper.Config.html#suppress_otp_env/1","title":"Grouper.Config.suppress_otp_env/1","type":"function"},{"doc":"uniform data layer for groups of processes","ref":"Grouper.Data.html","title":"Grouper.Data","type":"module"},{"doc":"identifies which type of group is being used, builds API functions, and caches them in the process dictionaryOptions:leader - override detected group leader with specified one (mostly used in testing)","ref":"Grouper.Data.html#api/1","title":"Grouper.Data.api/1","type":"function"},{"doc":"delete data value of a given type and a given key","ref":"Grouper.Data.html#del/3","title":"Grouper.Data.del/3","type":"function"},{"doc":"enumerates all key-values of a given data typeSpecify the special type :_ to read data of all types","ref":"Grouper.Data.html#enum/2","title":"Grouper.Data.enum/2","type":"function"},{"doc":"get data value of a given type and a given key","ref":"Grouper.Data.html#get/3","title":"Grouper.Data.get/3","type":"function"},{"doc":"store data value of a given type and a given key","ref":"Grouper.Data.html#put/4","title":"Grouper.Data.put/4","type":"function"},{"doc":"","ref":"Grouper.Data.html#t:key/0","title":"Grouper.Data.key/0","type":"type"},{"doc":"","ref":"Grouper.Data.html#t:opts/0","title":"Grouper.Data.opts/0","type":"type"},{"doc":"","ref":"Grouper.Data.html#t:t/0","title":"Grouper.Data.t/0","type":"type"},{"doc":"","ref":"Grouper.Data.html#t:type/0","title":"Grouper.Data.type/0","type":"type"},{"doc":"","ref":"Grouper.Data.html#t:value/0","title":"Grouper.Data.value/0","type":"type"},{"doc":"data layer driver for :application implied groups","ref":"Grouper.Data.App.html","title":"Grouper.Data.App","type":"module"},{"doc":"delete data value of a given type and a given key","ref":"Grouper.Data.App.html#del/3","title":"Grouper.Data.App.del/3","type":"function"},{"doc":"enumerates all key-values of a given typeSpecify the special type :_ to read data of all types","ref":"Grouper.Data.App.html#enum/2","title":"Grouper.Data.App.enum/2","type":"function"},{"doc":"get data value of a given type and a given key","ref":"Grouper.Data.App.html#get/3","title":"Grouper.Data.App.get/3","type":"function"},{"doc":"initialize application-driven dataThis may load OTP environment data. It only attempts to either load or suppress loading the first time it's called. Given that loading is the default, it is necessary to suppress loading the environment very early in application loading if this is desired.Optionsload_otp_env: Set to true to load the current application's environment (the default), to an atom to load a specific applications environment, or to false to suppress loading any environment.","ref":"Grouper.Data.App.html#init/2","title":"Grouper.Data.App.init/2","type":"function"},{"doc":"store data value of a given type and a given key","ref":"Grouper.Data.App.html#put/4","title":"Grouper.Data.App.put/4","type":"function"},{"doc":"","ref":"Grouper.Data.App.html#t:app_name/0","title":"Grouper.Data.App.app_name/0","type":"type"},{"doc":"","ref":"Grouper.Data.App.html#t:key/0","title":"Grouper.Data.App.key/0","type":"type"},{"doc":"","ref":"Grouper.Data.App.html#t:meta/0","title":"Grouper.Data.App.meta/0","type":"type"},{"doc":"","ref":"Grouper.Data.App.html#t:type/0","title":"Grouper.Data.App.type/0","type":"type"},{"doc":"","ref":"Grouper.Data.App.html#t:value/0","title":"Grouper.Data.App.value/0","type":"type"},{"doc":"data layer driver for explicitly-created groups","ref":"Grouper.Data.Group.html","title":"Grouper.Data.Group","type":"module"},{"doc":"delete data value of a given type and a given key","ref":"Grouper.Data.Group.html#del/3","title":"Grouper.Data.Group.del/3","type":"function"},{"doc":"enumerates all key-values of a given typeSpecify the special type :_ to read data of all types","ref":"Grouper.Data.Group.html#enum/2","title":"Grouper.Data.Group.enum/2","type":"function"},{"doc":"get data value of a given type and a given key","ref":"Grouper.Data.Group.html#get/3","title":"Grouper.Data.Group.get/3","type":"function"},{"doc":"initialize group-driven dataCurrently does nothing.","ref":"Grouper.Data.Group.html#init/2","title":"Grouper.Data.Group.init/2","type":"function"},{"doc":"store data value of a given type and a given key","ref":"Grouper.Data.Group.html#put/4","title":"Grouper.Data.Group.put/4","type":"function"},{"doc":"","ref":"Grouper.Data.Group.html#t:group_pid/0","title":"Grouper.Data.Group.group_pid/0","type":"type"},{"doc":"","ref":"Grouper.Data.Group.html#t:group_tid/0","title":"Grouper.Data.Group.group_tid/0","type":"type"},{"doc":"","ref":"Grouper.Data.Group.html#t:key/0","title":"Grouper.Data.Group.key/0","type":"type"},{"doc":"","ref":"Grouper.Data.Group.html#t:meta/0","title":"Grouper.Data.Group.meta/0","type":"type"},{"doc":"","ref":"Grouper.Data.Group.html#t:type/0","title":"Grouper.Data.Group.type/0","type":"type"},{"doc":"","ref":"Grouper.Data.Group.html#t:value/0","title":"Grouper.Data.Group.value/0","type":"type"},{"doc":"Starting a group turns whatever is starting it into a &quot;group&quot;. This is typically either run from a standalone process or under a supervisor as the first child. It is not usually necessary under an Application, as the application itself is used as a group identifier.In either case, it will &quot;commandeer&quot; the group-leader setting of that calling process, making all subsequent processes started use it as leader which is necessary for Grouper's functionality to work.StandaloneTypically run in tests or scripts,In the rare event that two groups must run under the same supervisor, it is possible to specify a &quot;group key&quot; to distinguish the two groups.ApplicationWhen running under an application, it's assumed to be your group. As a convenience, the OTP environmentAttempting to start a group under an application will return a :group_under_app error by default. Forcing a group to run under a supervisor can break OTP shutdown behavior and is almost always a terrible idea.Restart BehaviorWhen run under a supervisor, a group may be restarted automatically should it fail. A flag is set in the supervisor's process dictionary that is used to detect this.Since death of a group_leader is fairly catastrophic, this process will not try to restart it. Rather, all subsequent attempts will automatically fail with a :group_leader_died error.This should fail the supervisor fairly quickly which will, in turn, more properly clean up the whole tree. Given that group leaders very rarely fail, this shouldn't be a common occurence.Options:group_key - when more than one group is running under a single supervisor, this key is used to disambiguate start and stop requests.:force_group - when set to true, will start a group under an application. This is dangerous as it interferes with OTP's application shutdown behavior.Additional options are passed on to the Grouper.GroupLeader.start_link/1 function.","ref":"Grouper.Group.html","title":"Grouper.Group","type":"module"},{"doc":"provides instructions for supervisors to run a group process","ref":"Grouper.Group.html#child_spec/1","title":"Grouper.Group.child_spec/1","type":"function"},{"doc":"initialize this process and its descendants to be their own group","ref":"Grouper.Group.html#start_link/1","title":"Grouper.Group.start_link/1","type":"function"},{"doc":"deactivates group behavior for this process","ref":"Grouper.Group.html#stop/1","title":"Grouper.Group.stop/1","type":"function"},{"doc":"This implements a group leader capable of IO and storing group metadata.A Grouper.GroupLeader forwards IO upstream, just like any other group_leader. In addition, it also allocates a ETS table for this group and registers it in a global ETS table.Requests for configuration and naming functions (mediated by the Grouper.Data module) store information in this ETS table (among other places).","ref":"Grouper.GroupLeader.html","title":"Grouper.GroupLeader","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.See Supervisor.","ref":"Grouper.GroupLeader.html#child_spec/1","title":"Grouper.GroupLeader.child_spec/1","type":"function"},{"doc":"gets state data for a given group leader","ref":"Grouper.GroupLeader.html#get_group_data/1","title":"Grouper.GroupLeader.get_group_data/1","type":"function"},{"doc":"gets original group leader for this process","ref":"Grouper.GroupLeader.html#get_group_leader/1","title":"Grouper.GroupLeader.get_group_leader/1","type":"function"},{"doc":"Starts a Grouper.Leader process linked to the current process.This is typically used in a supervision tree.In addition to the normal options accepted by GenServer.start_link/3, this also accepts the following options:commandeer - during initialization, sets itself as group_leader for this process or list of processes instead of the calling processleader - during initialization, sets this process as the group_leader to which it forwards IO requestsparent - this specifies the parent process, useful for testing and when not being started in a normal supervision tree","ref":"Grouper.GroupLeader.html#start_link/1","title":"Grouper.GroupLeader.start_link/1","type":"function"},{"doc":"stops a group leader process","ref":"Grouper.GroupLeader.html#stop/2","title":"Grouper.GroupLeader.stop/2","type":"function"},{"doc":"valid info terms","ref":"Grouper.GroupLeader.html#t:infos/0","title":"Grouper.GroupLeader.infos/0","type":"type"},{"doc":"keyword options for init function","ref":"Grouper.GroupLeader.html#t:init_option/0","title":"Grouper.GroupLeader.init_option/0","type":"type"},{"doc":"options to init function","ref":"Grouper.GroupLeader.html#t:init_options/0","title":"Grouper.GroupLeader.init_options/0","type":"type"},{"doc":"IO request info term","ref":"Grouper.GroupLeader.html#t:io_request/0","title":"Grouper.GroupLeader.io_request/0","type":"type"},{"doc":"keyword options for start function","ref":"Grouper.GroupLeader.html#t:option/0","title":"Grouper.GroupLeader.option/0","type":"type"},{"doc":"options to start function","ref":"Grouper.GroupLeader.html#t:options/0","title":"Grouper.GroupLeader.options/0","type":"type"},{"doc":"state for Grouper.Leader","ref":"Grouper.GroupLeader.html#t:t/0","title":"Grouper.GroupLeader.t/0","type":"type"},{"doc":"Functions for identifying group leader types.","ref":"Grouper.Ident.html","title":"Grouper.Ident","type":"module"},{"doc":"Identifies the type of group leader for current process.","ref":"Grouper.Ident.html#identify_group_leader/0","title":"Grouper.Ident.identify_group_leader/0","type":"function"},{"doc":"Identifies the type of group leader by its pid.","ref":"Grouper.Ident.html#identify_group_leader/1","title":"Grouper.Ident.identify_group_leader/1","type":"function"},{"doc":"","ref":"Grouper.Ident.html#t:app_leader/0","title":"Grouper.Ident.app_leader/0","type":"type"},{"doc":"","ref":"Grouper.Ident.html#t:app_meta/0","title":"Grouper.Ident.app_meta/0","type":"type"},{"doc":"","ref":"Grouper.Ident.html#t:capture_io_leader/0","title":"Grouper.Ident.capture_io_leader/0","type":"type"},{"doc":"","ref":"Grouper.Ident.html#t:capture_io_meta/0","title":"Grouper.Ident.capture_io_meta/0","type":"type"},{"doc":"possible errors identifying group leaders","ref":"Grouper.Ident.html#t:group_leader_errors/0","title":"Grouper.Ident.group_leader_errors/0","type":"type"},{"doc":"","ref":"Grouper.Ident.html#t:grouper_leader/0","title":"Grouper.Ident.grouper_leader/0","type":"type"},{"doc":"","ref":"Grouper.Ident.html#t:grouper_meta/0","title":"Grouper.Ident.grouper_meta/0","type":"type"},{"doc":"type of group leader process","ref":"Grouper.Ident.html#t:leader_spec/0","title":"Grouper.Ident.leader_spec/0","type":"type"},{"doc":"","ref":"Grouper.Ident.html#t:shell_leader/0","title":"Grouper.Ident.shell_leader/0","type":"type"},{"doc":"","ref":"Grouper.Ident.html#t:shell_meta/0","title":"Grouper.Ident.shell_meta/0","type":"type"},{"doc":"","ref":"Grouper.Ident.html#t:user_leader/0","title":"Grouper.Ident.user_leader/0","type":"type"},{"doc":"","ref":"Grouper.Ident.html#t:user_meta/0","title":"Grouper.Ident.user_meta/0","type":"type"},{"doc":"Reaps metadata for defunct groups and registered names.","ref":"Grouper.Reaper.html","title":"Grouper.Reaper","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.See Supervisor.","ref":"Grouper.Reaper.html#child_spec/1","title":"Grouper.Reaper.child_spec/1","type":"function"},{"doc":"starts reaper","ref":"Grouper.Reaper.html#start_link/1","title":"Grouper.Reaper.start_link/1","type":"function"},{"doc":"Provides name registration functions in the style of :global and Registry. Usually used with GenServer.start_link/3 using :via option to provide an isolated namespace for various processes.","ref":"Grouper.Registry.html","title":"Grouper.Registry","type":"module"},{"doc":"registers a process under a name within a groupOptionsOptions are passed on to the underlying data layer. See Data.api/1 for details.","ref":"Grouper.Registry.html#register_name/3","title":"Grouper.Registry.register_name/3","type":"function"},{"doc":"sends a message to a process under a name within a groupOptionsOptions are passed on to the underlying data layer. See Data.api/1 for details.","ref":"Grouper.Registry.html#send/3","title":"Grouper.Registry.send/3","type":"function"},{"doc":"unregisters a process under a name within a groupOptionsOptions are passed on to the underlying data layer. See Data.api/1 for details.","ref":"Grouper.Registry.html#unregister_name/2","title":"Grouper.Registry.unregister_name/2","type":"function"},{"doc":"finds a process under a name within a groupOptionsOptions are passed on to the underlying data layer. See Data.api/1 for details.","ref":"Grouper.Registry.html#whereis_name/2","title":"Grouper.Registry.whereis_name/2","type":"function"},{"doc":"supervisor process for Grouper application","ref":"Grouper.Supervisor.html","title":"Grouper.Supervisor","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.See Supervisor.","ref":"Grouper.Supervisor.html#child_spec/1","title":"Grouper.Supervisor.child_spec/1","type":"function"},{"doc":"Starts application-level supervisor for Grouper.Maintains global information that drives groups.","ref":"Grouper.Supervisor.html#start_link/2","title":"Grouper.Supervisor.start_link/2","type":"function"},{"doc":"no group errorUsed by immediate-mode (i.e. !-suffixed) versions of functions.","ref":"Grouper.NoGroupError.html","title":"Grouper.NoGroupError","type":"exception"},{"doc":"Callback implementation for Exception.message/1.","ref":"Grouper.NoGroupError.html#message/1","title":"Grouper.NoGroupError.message/1","type":"function"},{"doc":"GrouperIsolates groups of process subtrees together for configuration and name registration purposes.Do you struggle with Elixir's global namespace for process names?Do all of your tests run synchronously because of this collisions?Do you mutate your application environment during tests?If the above problems sounds familiar, Grouper might be for you.","ref":"readme.html","title":"Overview","type":"extras"},{"doc":"Simply start your GenServers like this:GenServer.start_link(mod, arg, name: {:via, Grouper.Registry})And access configuration like this:Grouper.Config.get(:key)In tests, scripts and IEX; initialize a group with:{:ok, _} = Grouper.start_link()Or run a single function in its own group like this:Grouper.exec!(&amp;MyApp.my_task/0)During normal application runtime, each application gets its own namespace for processes and has isolated config.During tests, however, each test can get its own group with isolated naming and configuration. This makes it trivial to run all of your tests asynchronously, eliminates the need to use global config for things like mocking, and prevents config mutations in different tests from interfering with each other.Scripts and IEX can similarly benefit from group isolation with a single command, thereby rounding out behavior to be identical in all common execution environments.","ref":"readme.html#usage","title":"Overview - Usage","type":"extras"},{"doc":"For convenience, the OTP config environment is loaded into the config for you, simplifying migration from older, global configuration. This can be suppressed if desired (see Grouper.Config.suppress_otp_env/1 for details).","ref":"readme.html#migration","title":"Overview - Migration","type":"extras"},{"doc":"This package is available in Hex. The package can be installed by adding grouper to your list of dependencies in mix.exs:def deps do [ {:grouper, &quot;~&gt; 0.1.0&quot;} ] endDocumentation is generated with ExDoc and published on HexDocs. The docs can be found at https://hexdocs.pm/grouper.","ref":"readme.html#installation","title":"Overview - Installation","type":"extras"},{"doc":"MIT LicenseCopyright (c) 2021 Jayson VantuylPermission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the &quot;Software&quot;), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.THE SOFTWARE IS PROVIDED &quot;AS IS&quot;, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html","title":"License","type":"extras"}]